\name{cem}
\alias{cem}
\title{Coarsened Exact Matching}
\description{
Implementation of Coarsened Exact Matching
}
\usage{
cem(treatment = NULL, data, cutpoints = NULL, drop = NULL, 
    eval.imbalance = FALSE, k2k = FALSE,  method=NULL, mpower=2, verbose = 0)
}
\arguments{
  \item{treatment}{character, name of the treatment variable}
  \item{data}{a data.frame}
  \item{cutpoints}{named list each describing the cutpoints for the
  variables (the names are variable names).
  Each list element is either a vector of cutpoints, a number of
  cutpoints, or
  a method for automatic bin contruction. See Details.}
  \item{drop}{a vector of variable names in the data frame to ignore
    during matching}
  \item{eval.imbalance}{Boolean. See Details.}
  \item{k2k}{boolean, return k-to-k matching? Default = \code{FALSE}}
  \item{method}{distance method to use in \code{k2k} matching. See Details.}
  \item{mpower}{power of the Minkowski distance. See Details.}
  \item{verbose}{controls level of verbosity. Default=0.}
}
\details{
When specifying cutpoints, several automatic methods can be chosen among
``\code{sturges}'' (Surges' rule, the default),
``\code{fd}'' (Freedman-Diaconis' rule), ``\code{scott}'' 
(Scott's rule) and ``ss'' (Shimazaki-Shinomoto's rule). 
See references for a description of each rule.


\code{verbose}: a number greater or equal to 0. The higher, the 
more info are provided during the execution of the algorithm. 


If \code{eval.imbalance} = \code{TRUE}, \code{cem$imbalance} contains the
imbalance measure by absolute difference in means for numerical variables 
and chi-square distance for categorical variables. If \code{FALSE} (the default)
then \code{cem$imbalance} is set to \code{NULL}.

If \code{k2k} is set to \code{TRUE}, the algorithm return strata with the same
number of treated and control units per stratum, otherwise all the matched units
are returned (default). When \code{k2k} = \code{TRUE}, the user can choose a 
\code{method} (between `\code{euclidean}', `\code{maximum}', `\code{manhattan}',
`\code{canberra}', `\code{binary}' and `\code{minkowski}') for nearest neighbor
matching inside each \code{cem} strata. By default \code{method} is set to 
`\code{NULL}', which means random matching inside \code{cem} strata. For the 
Minkowski distance the power can be specified via the argument \code{mpower}'.
For more information on \code{method != NULL}, refer to \code{\link{dist}}
help page.

In case of missing data, \code{cem} gives a warning and treats missing values 
as distinct values and match observations with missing values in the same 
variable in the same stratum provided that all the remaining (corasened) 
covariates match.
}
\value{
  \item{call}{the call}
  \item{strata}{vector of stratum number in which each observation belongs, 
				NA if the observation has not been matched}
  \item{n.strata}{number of strata generated}
  \item{vars}{report variables names used for the match}
  \item{drop}{variables removed from the match}
  \item{breaks}{named list of cutpoints, eventually NULL}
  \item{treatment}{name of the treatment variable}
  \item{groups}{factor, each observation belong to one group generated by the 
				treatment variable}
  \item{n.groups}{number of groups identified by the treatment variable}
  \item{group.idx}{named list, index of observations belonging to each group}
  \item{group.len}{sizes of groups}
  \item{tab}{summary table of matched by group}
  \item{imbalance}{NULL or a vector of imbalances. See Details.}
}
\references{Stefano Iacus, Gary King, Giusseppe Porro, ``Matching for
  Casual Inference Without Balance Checking,''
  http://gking.harvard.edu/files/abs/cem-abs.shtml}
\author{Stefano Iacus, Gary King, and Giusseppe Porro}
\examples{
data(LL)

mybr = list(re74=hist(LL$re74,plot=FALSE)$breaks, 
 re75 = hist(LL$re75,plot=FALSE)$breaks,
 age = hist(LL$age,plot=FALSE)$breaks,
 education = hist(LL$education,plot=FALSE)$breaks)
   
L1.meas(LL$treated, LL[,-c(1,9)], breaks=mybr)
eval.match(LL$treated, LL[,-c(1,9)], breaks=mybr)

# cem match: automatic bin choice
mat <- cem(treatment="treated",data=LL, drop="re78")
mat$tab
cem1.idx <- which(mat$matched)
# imbalance
L1.meas(LL$treated[cem1.idx], LL[cem1.idx,-c(1,9)], breaks=mybr)
eval.match(LL$treated[cem1.idx], LL[cem1.idx,-c(1,9)], breaks=mybr)
L1.meas(LL$treated[cem1.idx], LL[cem1.idx,-c(1,9)], breaks=mybr,weights=mat$w[cem1.idx])
eval.match(LL$treated[cem1.idx], LL[cem1.idx,-c(1,9)], breaks=mybr,weights=mat$w[cem1.idx])


# cem match: user choiced coarsening
re74cut <- hist(LL$re74, br=seq(0,max(LL$re74)+1000, by=1000),plot=FALSE)$breaks
re75cut <- hist(LL$re75, br=seq(0,max(LL$re75)+1000, by=1000),plot=FALSE)$breaks
agecut <- hist(LL$age, br=seq(15,55, length=14),plot=FALSE)$breaks
mycp <- list(re75=re75cut, re74=re74cut, age=agecut)
mat <- cem(treatment="treated",data=LL, drop="re78",cutpoints=mycp)
mat$tab
cem2.idx <- which(mat$matched)
#imbalance
L1.meas(LL$treated[cem2.idx], LL[cem2.idx,-c(1,9)], breaks=mybr)
eval.match(LL$treated[cem2.idx], LL[cem2.idx,-c(1,9)], breaks=mybr)
L1.meas(LL$treated[cem2.idx], LL[cem2.idx,-c(1,9)], breaks=mybr,weights=mat$w[cem2.idx])
eval.match(LL$treated[cem2.idx], LL[cem2.idx,-c(1,9)], breaks=mybr,weights=mat$w[cem2.idx])

# cem match: user choiced coarsening, k-to-k matching
mat <- cem(treatment="treated",data=LL, drop="re78",cutpoints=mycp,k2k=TRUE)
mat$tab
cem3.idx <- which(mat$matched)
#imbalance
L1.meas(LL$treated[cem3.idx], LL[cem3.idx,-c(1,9)], breaks=mybr)
eval.match(LL$treated[cem3.idx], LL[cem3.idx,-c(1,9)], breaks=mybr)
L1.meas(LL$treated[cem3.idx], LL[cem3.idx,-c(1,9)], breaks=mybr,weights=mat$w[cem3.idx])
eval.match(LL$treated[cem3.idx], LL[cem3.idx,-c(1,9)], breaks=mybr,weights=mat$w[cem3.idx])

# mahalnobis matching
require(MatchIt)
mah <- matchit(treated~age+education+re74+re75+black+hispanic+nodegree+married+u74+u75,
   distance="mahalanobis", data=LL)
mah
idx1 <- as.numeric(mah$match.matrix)
idx2 <- as.numeric(rownames(mah$match.matrix))
mah.idx <- match( c(idx1,idx2), rownames(LL))
#imbalance
L1.meas(LL$treated[mah.idx], LL[mah.idx,-c(1,9)], breaks=mybr)
eval.match(LL$treated[mah.idx], LL[mah.idx,-c(1,9)], breaks=mybr)
}
\keyword{multivariate}
\keyword{datagen}


